\name{SimComp-package}
\alias{SimComp-package}
\alias{SimComp}
\docType{package}
\title{
Simultaneous Comparisons for Multiple Endpoints
}
\description{
This package provides simultaneous tests and confidence
intervals for one-way experimental designs with one or
many normally distributed, primary response variables
(endpoints). Means of several groups or dose levels can
be compared
\itemize{
  \item by arbitrary contrasts, like the Dunnett or the
    Tukey test,
  \item for balanced or unbalanced sample sizes,
  \item for a single endpoint or for many endpoints
    simultaneously,
  \item for homogeneous or heterogeneous variances/
    covariance matrices of the groups, and
  \item in terms of differences or ratios.
  }
Exact or approximate multivariatet-distributions,
respectively, are used for quantiles or p-values.

For example, the well-known conventional all-pair
comparison of Tukey (1953) can be performed by
specifying only a single endpoint and homogeneous (co-)
variances. On the other hand, it's also possible to do
the same, but for many endpoints simultaneously, with
heterogeneous covariance matrices and in terms of
ratios.

For multiple comparisons of means of heteroscedastic
data, see Hasler and Hothorn (2008). The test procedure
for multiple endpoints is described by Hasler (2009).
}
\details{
\tabular{ll}{
Package: \tab SimComp\cr
Type: \tab Package\cr
Version: \tab 1.4.3\cr
Date: \tab 2009-11-05\cr
License: \tab GPL\cr
LazyLoad: \tab yes\cr
}
}
\author{
Mario Hasler

Maintainer: Mario Hasler <hasler@email.uni-kiel.de>
}
\references{
Hasler, M. (2009): Extensions of Multiple Contrast Tests.
\emph{PhD Thesis}, Gottfried-Wilhelm-Leibniz-Universitaet
Hannover.

Hasler, M. and Hothorn, L.A. (submitted): A Dunnett-type
Procedure for Multiple Endpoints

Hasler, M. and Hothorn, L.A. (2008): Multiple contrast
tests in the presence of heteroscedasticity.
\emph{Biometrical Journal} 50, 793-800.
}
\keyword{ package }
\seealso{ \code{\link[mratios:mratios-package]{mratios}} }
\examples{
# Example 1:
# A Dunnett-test for the groups B and H against the standard S, on
# the (single) endpoint Thromb.count, assuming unequal variances for
# the groups. This is the well-known Dunnett-test but in the
# presence of heteroscedasticity.

data(coagulation)

comp1 <- SimTestDiff(data=coagulation, grp="Group", resp="Thromb.count", type="Dunnett", base=3, alternative="greater", covar.equal=FALSE)
comp1

# Example 2:
# A Dunnett-test for the groups B and H against the standard S,
# simultaneously on all endpoints, assuming unequal covariance
# matrices for the groups.

data(coagulation)

comp2 <- SimTestDiff(data=coagulation, grp="Group", resp=c("Thromb.count","ADP","TRAP"), type="Dunnett", alternative="greater", covar.equal=FALSE)
summary(comp2)
}
